#!/bin/env python3

import sys

print(sys.path)

import pymars2grib

encoder = pymars2grib.Mars2Grib()

mars = {
    "origin": "ecmf",
    "class": "od",
    "stream": "oper",
    "type": "fc",
    "expver": "0001",
    "grid": "N200",
    "packing": "ccsds",
    "param": 130,
    "levtype": "hl",
    "levelist": 2,
    "date": 20260205,
    "time": 000000,
    "step": 0,
}

vals = [237.15] * 200

encoder.encode(vals, mars)



#!/usr/bin/env python3

# (c) Copyright 1996-2012 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# Documentation is available on Confluence
# https://confluence.ecmwf.int/display/METK/MetKit+configuration


import mysql.connector
import yaml
import re
import os

connectionDetails = dict(
	host = "webapps-db-prod",
	user = os.environ['PARAM_DB_USER'],
	passwd = os.environ['PARAM_DB_PASSWORD'])

def execute(database, fun, prefix=None, asList=False, lowercase=True):
	connection = mysql.connector.connect(**connectionDetails)
	cursor = connection.cursor()
	cursor.execute("USE " + database)
	r = fun(cursor, prefix, asList, lowercase) if (prefix) else fun(cursor)
	cursor.close()
	connection.close()
	return r

################################################################################################

def ids(cursor, prefix, asList, lowercase):

	PRODGEN = {}
	if os.path.exists("prodgen-" + prefix + "ids.yaml"):
		with open("prodgen-" + prefix + "ids.yaml") as f:
			PRODGEN = yaml.load(f.read(), Loader=yaml.FullLoader)

	IDS = {}
	if os.path.exists(prefix + "ids.yaml"):
		with open(prefix + "ids.yaml") as f:
			IDS = yaml.load(f.read(), Loader=yaml.FullLoader)
			if asList:
				# Convert list of lists to dictionary
				IDS = {item[0]: item[1:] for item in IDS}

	cursor.execute("select * from " + prefix)

	for data in cursor.fetchall():
		paramid, abbr, longname = int(data[0]), data[1], data[2]
		if lowercase:
			abbr = abbr.lower()

		abbr = re.sub(r"\W", "_", abbr)
		abbr = re.sub(r"_+", "_", abbr)
		abbr = re.sub(r"^_", "", abbr)
		abbr = re.sub(r"_$", "", abbr)

		if not abbr:
			abbr = "_%s_%06d" % (prefix, paramid,)

		entry = [abbr.strip(), longname.strip()]

		if paramid in PRODGEN:
			pgen = [str(x).lower() for x in PRODGEN[paramid]]
			p = []
			for n in pgen:
				if (
					n not in entry
				):  #  and (' ' not in n) and ('.' not in n): #  and ('-' not in n):
					entry.append(n)
					p.append(n)

		entry = tuple(entry)

		if entry[0].lower() == entry[1].lower():
			entry = tuple([entry[0]])
		else:
			entry = tuple(entry)

		if paramid in IDS:
			before = tuple(IDS[paramid])
			if before != entry:
				print(
					"WARNING! updated {}id: {},  {} => {}".format(prefix,  paramid, before, entry)
				)
				IDS[paramid] = list(entry)
		else:
			print("new {}id: {} {}".format(prefix, paramid, entry))
			IDS[paramid] = list(entry)

	for paramid, entry in PRODGEN.items():
		if paramid not in IDS:
			print("WARNING! adding pseudo-{}id: {},  {}".format(prefix, paramid, tuple(entry)))
			IDS[paramid] = entry

	if asList:
		# Sort by key and convert to list of lists
		return [[k] + v for k, v in sorted(IDS.items(), key=lambda item: item[0])]
	return IDS


################################################################################################

def getListOfRT(cursor, group_id):
	def sorted(cursor):
		cursor.execute("SELECT code FROM odbadmin_reporttype WHERE group_id = " + str(group_id))
		r = [r[0] for r in cursor.fetchall()]
		r.sort()
		return r
	return sorted(cursor)

T = """- context:\n    type: [ofb,mfb,oai,sfb,fsoifb,fcdfb,ofa]\n    obsgroup: [%(OBSGROUP)s]\n  defaults:\n    reportype: [%(REPORTYPE_LIST)s]\n"""

def reportype(cursor):
	cursor.execute("SELECT id,marsname FROM odbadmin_group")
	GROUP = dict([(r[0],str(r[1]).lower()) for r in cursor.fetchall()])

	cursor.execute("SELECT DISTINCT group_id FROM odbadmin_reporttype ORDER BY group_id")
	groups = [r[0] for r in cursor.fetchall()]
	def expand(g):
		return T % dict(OBSGROUP = GROUP[g], REPORTYPE_LIST = "/".join([str(r) for r in getListOfRT(cursor, g)]))
	return "\n".join([expand(group_id) for group_id in groups])

################################################################################################

def header(f,o):
	f.write(
		"---\n# File automatically generated by %s --%s\n# Do not edit\n\n"
		% (os.path.basename(__file__), o)
	)

options = 'chem param reportype'.split()

def main():
	import sys
	opts = sys.argv[1:]
	if len(opts) != 1 or opts[0].lstrip('-') not in options:
		print(sys.argv[0] + ": Possible options: --" + ", --".join(options))
		sys.exit(1)
	o = opts[0].lstrip('-')
	match o:
		case 'chem':
			content = execute("param", ids, o, True, False)
			with open(o + "ids.yaml", "w") as f:
				header(f, o)
				yaml.safe_dump(content, f, default_flow_style=None)
		case 'param':
			content = execute("param", ids, o)
			with open(o + "ids.yaml", "w") as f:
				header(f, o)
				yaml.safe_dump(content, f, default_flow_style=False)
		case 'reportype':
			with open("reportype.yaml", "w") as f:
				header(f, o)
				f.write(execute("odbgov", reportype))

if __name__ == "__main__":
	main()



from .mars2grib_core import init_bindings, Mars2GribCore

class Mars2Grib():
    """
    Mars2Grib encoder object
    """

    def __init__(self, options: dict = {}):
        init_bindings()
        self._mars2grib = Mars2GribCore(options)

    def encode(self, values: list[float], mars: dict, misc: dict = {}) -> bytes:
        """
        Encode values, described by mars and (optionally) misc dictionaries, as a GRIB2 message

        :param values: Values to encode
        :type values: list[float]
        :param mars: Mars keys describing the values
        :type mars: dict
        :param misc: Miscellaneous (non-mars) keys describing the values, may be empty
        :type misc: dict
        :return: Encoded GRIB2 message as bytes object
        :rtype: bytes
        """
        return self._mars2grib.encode(values, mars, misc)



import os
from cffi import FFI
import findlibs
from typing import IO, Iterator
import warnings
from ._version import __version__

ffi = FFI()


def ffi_encode(data) -> bytes:
    if isinstance(data, bytes):
        return data

    if not isinstance(data, str):
        data = str(data)

    return data.encode(encoding="utf-8", errors="surrogateescape")


def ffi_decode(data: FFI.CData) -> str:
    buf = ffi.string(data)
    if isinstance(buf, str):
        return buf
    else:
        return buf.decode(encoding="utf-8", errors="surrogateescape")


class MarsRequest:
    def __init__(self, verb: str | None = None, **kwargs):
        """
        Create MetKit MarsRequest object. Parameters and values in
        the request can be specified through kwargs, noting that
        reserved words in Python must be suffixed with "_" e.g. "class_"
        """
        crequest = ffi.new("metkit_marsrequest_t **")
        lib.metkit_marsrequest_new(crequest)
        self.__request = ffi.gc(crequest[0], lib.metkit_marsrequest_delete)
        if verb is not None:
            lib.metkit_marsrequest_set_verb(self.__request, ffi_encode(verb))
        for param, values in kwargs.items():
            self[param.rstrip("_")] = values

    def ctype(self) -> FFI.CData:
        return self.__request

    def verb(self) -> str:
        cverb = ffi.new("const char **")
        lib.metkit_marsrequest_verb(self.__request, cverb)
        return ffi_decode(cverb[0])

    def expand(self, inherit: bool = True, strict: bool = False) -> "MarsRequest":
        """
        Return expanded request

        Params
        ------
        inherit: bool, if True, populates expanded request with default values
        strict: bool, if True, raise error instead of warning for invalid values

        Returns
        -------
        Request, resulting from expansion
        """
        expanded_request = MarsRequest()
        lib.metkit_marsrequest_expand(
            self.__request, inherit, strict, expanded_request.ctype()
        )
        return expanded_request

    def validate(self):
        """
        Check if request is valid against MARS language definition. Does not
        inherit missing parameters.

        Raises
        ------
        Exception if request is incompatible with MARS language definition
        """
        self.expand(False, True)

    def keys(self) -> Iterator[str]:
        """
        Get iterator over parameters in request

        Returns
        -------
        Iterator over parameter names
        """
        it_c = ffi.new("metkit_paramiterator_t **")
        lib.metkit_marsrequest_params(self.__request, it_c)
        it = ffi.gc(it_c[0], lib.metkit_paramiterator_delete)
        
        while lib.metkit_paramiterator_next(it) == lib.METKIT_ITERATOR_SUCCESS:
            cparam = ffi.new("const char **")
            lib.metkit_paramiterator_current(it, cparam)
            param = ffi_decode(cparam[0])
            yield param

    def num_values(self, param: str) -> int:
        """
        Number of values for parameter

        Params
        ------
        param: parameter name

        Returns
        -------
        int
        """
        cparam = ffi_encode(param)
        count = ffi.new("size_t *", 0)
        lib.metkit_marsrequest_count_values(self.__request, cparam, count)
        return count[0]

    def merge(self, other: "MarsRequest") -> "MarsRequest":
        """
        Merge the values in another request with existing request and returns result as a
        new Request object. Does not modify inputs to merge. Both input requests must contain
        the same values and the resulting request object must be compatible with MARS language
        definition

        Params
        ------
        other: Request, request to merge with self

        Returns
        -------
        Request, containing the result of the merge

        Raises
        ------
        ValueError if parameters in the two requests do not match
        MetKitException if resulting request is not compatible with MARS language definition
        """
        if set(self.keys()) != set(other.keys()):
            raise ValueError("Can not merge requests with different parameters.")
        res = MarsRequest(self.verb(), **{k: v for k, v in self})
        lib.metkit_marsrequest_merge(res.ctype(), other.ctype())
        res.validate()
        return res

    def __iter__(self) -> Iterator[tuple[str, list[str]]]:
        for param in self.keys():
            yield param, self[param]

    def __getitem__(self, param: str) -> str | list[str]:
        nvalues = self.num_values(param)
        values = []
        for index in range(nvalues):
            cvalue = ffi.new("const char **")
            lib.metkit_marsrequest_value(self.__request, ffi_encode(param), index, cvalue)
            value = ffi_decode(cvalue[0])
            if nvalues == 1:
                return value
            values.append(value)
        return values

    def __contains__(self, param: str) -> bool:
        has = ffi.new("bool *", False)
        lib.metkit_marsrequest_has_param(self.__request, ffi_encode(param), has)
        return has[0]

    def __setitem__(self, param: str, values: int | str | list[str]):
        if isinstance(values, (str, int)):
            values = [values]
        cvals = []
        for value in values:
            if isinstance(value, int):
                value = str(value)
            cvals.append(ffi.new("const char[]", value.encode("ascii")))
        lib.metkit_marsrequest_set(
            self.__request,
            ffi_encode(param),
            ffi.new("const char*[]", cvals),
            len(values),
        )

    def __eq__(self, other: "MarsRequest") -> bool:
        if self.verb() != other.verb():
            return False
        expanded = self.expand()
        other_expanded = other.expand()
        return dict(expanded) == dict(other_expanded)


def parse_mars_request(file_or_str: IO | str, strict: bool = False) -> list[MarsRequest]:
    """
    Function for parsing mars request from file object or string.

    Params
    ------
    file_or_str: string or file-like object, containing mars request
    strict: bool, whether to raise error or warning when request is not compatible with
    MARS language definition. In the case of warning, when False, the incompatible
    parameters are unset from the request.

    Returns
    -------
    list of Request
    """
    crequest_iter = ffi.new("metkit_requestiterator_t **")

    if isinstance(file_or_str, str):
        lib.metkit_parse_marsrequests(ffi_encode(file_or_str), crequest_iter, strict)
    else:
        lib.metkit_parse_marsrequests(
            ffi_encode(file_or_str.read()), crequest_iter, strict
        )
    request_iter = ffi.gc(crequest_iter[0], lib.metkit_requestiterator_delete)

    requests = []
    while lib.metkit_requestiterator_next(request_iter) == lib.METKIT_ITERATOR_SUCCESS:
        new_request = MarsRequest()
        lib.metkit_requestiterator_current(request_iter, new_request.ctype())
        requests.append(new_request)

    return requests


class MetKitException(RuntimeError):
    """Raised when MetKit library throws exception"""

    pass


class CFFIModuleLoadFailed(ImportError):
    """Raised when the shared library fails to load"""

    pass


class PatchedLib:
    """
    Patch a CFFI library with error handling

    Finds the header file associated with the MetKit C API and parses it,
    loads the shared library, and patches the accessors with
    automatic python-C error handling.
    """

    def __init__(self):
        libName = findlibs.find("metkit")

        if libName is None:
            raise RuntimeError("MetKit library not found")

        ffi.cdef(self.__read_header())
        self.__lib = ffi.dlopen(libName)

        # All of the executable members of the CFFI-loaded library are functions in the MetKit
        # C API. These should be wrapped with the correct error handling. Otherwise forward
        # these on directly.

        for f in dir(self.__lib):
            try:
                attr = getattr(self.__lib, f)
                setattr(
                    self, f, self.__check_error(attr, f) if callable(attr) else attr
                )
            except Exception as e:
                print(e)
                print("Error retrieving attribute", f, "from library")

        # Initialise the library, and set it up for python-appropriate behaviour

        self.metkit_initialise()

        # Check the library version

        versionstr = ffi.string(self.metkit_version()).decode("utf-8")
        if versionstr != __version__:
            warnings.warn(f"Metkit library version {versionstr} does not match python version {__version__}")

    def __read_header(self):
        with open(os.path.join(os.path.dirname(__file__), "metkit_c.h"), "r") as f:
            return f.read()

    def __check_error(self, fn, name: str):
        """
        If calls into the MetKit library return errors, ensure that they get
        detected and reported by throwing an appropriate python exception.
        """

        def wrapped_fn(*args, **kwargs):

            # debug
            retval = fn(*args, **kwargs)

            # Some functions dont return error codes. Ignore these.
            if name in ["metkit_version", "metkit_git_sha1"]:
                return retval

            # error codes:
            if retval not in (
                self.__lib.METKIT_SUCCESS,
                self.__lib.METKIT_ITERATOR_SUCCESS,
                self.__lib.METKIT_ITERATOR_COMPLETE,
            ):
                err = ffi_decode(self.__lib.metkit_get_error_string(retval))
                msg = "Error in function '{}': {}".format(name, err)
                raise MetKitException(msg)
            return retval

        return wrapped_fn


try:
    lib = PatchedLib()
except CFFIModuleLoadFailed as e:
    raise ImportError() from e



from datetime import datetime, timedelta
from contextlib import nullcontext as does_not_raise
import pytest

from pymetkit import parse_mars_request, MarsRequest, MetKitException

request = """
retrieve,
    class=od,
    domain=g,
    expver=0001,
    levtype=sfc,
    stream=enfo,
    date=-1,
    time=12,
    param=151.128,
    grid=O640,
    step=0/to/24/by/6,         
	target=test.grib,
	type=em
retrieve,
    class=od,
    domain=g,
    expver=0001,
    levtype=pl,
    stream=enfo,
    date=-1,
    time=12,
    param=129, 
    levelist=500,
    grid=O640,
    step=0/to/24/by/6,               
	target=test.grib,
	type=em
"""

yesterday = (datetime.today() - timedelta(days=1)).strftime("%Y%m%d")


def test_parse_file(tmpdir):
    request_file = f"{tmpdir}/requests"
    with open(request_file, "w") as f:
        f.write(request)
    requests = parse_mars_request(open(request_file, "r"))
    assert len(requests) == 2
    for req in requests:
        assert req.verb() == "retrieve"
        assert len(req["step"]) == 5
        assert req["date"] == yesterday
    assert "class" in requests[0]
    assert requests[1]["levelist"] == "500"

# @todo: [1] no longer raises an exception. Disable until METK-126 is resolved.
@pytest.mark.parametrize(
    "req_str, length, steps, strict, expectation",
    [
        [request, 2, 5, False, does_not_raise()],
        # [request, 2, 5, True, pytest.raises(MetKitException)],
        [
            "retrieve,class=od,date=-1,time=12,param=129,step=12,target=test.grib",
            1,
            1,
            False,
            does_not_raise(),
        ],
    ],
)
def test_parse_string(req_str, length, steps, strict, expectation):
    with expectation:
        requests = parse_mars_request(req_str, strict)
        assert len(requests) == length
        for req in requests:
            assert req.num_values("step") == steps


def test_empty_request(tmpdir):
    request_file = f"{tmpdir}/requests"
    with open(request_file, "w") as f:
        f.write("")
    requests = parse_mars_request(open(request_file, "r"))
    assert len(requests) == 0


def test_new_request():
    req = MarsRequest("retrieve")
    assert req.verb() == "retrieve"

    req = MarsRequest("request", class_="od", type="pf", date=["20200101", "20200102"])
    assert req["class"] == "od"
    assert req["type"] == "pf"
    assert req["date"] == ["20200101", "20200102"]


def test_request_from_expand():
    req = MarsRequest(
        "retrieve",
        **{
            "class": "od",
            "domain": "g",
            "date": "-1",
            "expver": "0001",
            "step": range(0, 13, 6),
        },
    )
    expanded = req.expand()
    assert expanded.verb() == req.verb()
    assert expanded["date"] == yesterday
    assert "param" in expanded
    expanded.validate()
    assert req == expanded

# @todo: [0] and [1] no longer raise an exception. Disable until METK-126 is resolved.
@pytest.mark.parametrize(
    "extra_kv",
    [
        # {"levelist": [500]},
        # {"type": "cf", "number": [1, 2]},
        {"class": "invalid"}
    ],
)
def test_request_validate(extra_kv):
    request = {
        "class": "od",
        "domain": "g",
        "date": "-1",
        "expver": "0001",
        "step": range(0, 13, 6),
        "levtype": "sfc",
    }
    request.update(extra_kv)
    req = MarsRequest("retrieve", **request)
    with pytest.raises(MetKitException):
        req.validate()


@pytest.mark.parametrize(
    "extra_kv, expectation",
    [
        [{"levtype": "pl", "date": "-1"}, pytest.raises(MetKitException)],
        [{"levtype": "sfc", "date": "-1", "type": "em"}, pytest.raises(ValueError)],
        [{"levtype": "sfc", "date": "20230101"}, does_not_raise()],
    ],
)
def test_request_merge(extra_kv, expectation):
    request = {
        "class": "od",
        "domain": "g",
        "expver": "0001",
        "step": range(0, 13, 6),
    }
    req = MarsRequest("retrieve", **request, date="-1", levtype="sfc")
    other_req = MarsRequest("retrieve", **request, **extra_kv)
    with expectation:
        req.merge(other_req)


@pytest.mark.parametrize(
    "verb, updates, expected",
    [["retrieve", {"date": 20230101, "param": 130}, True], ["compute", {}, False]],
)
def test_request_equality(verb, updates, expected):
    init_request = {
        "class": "od",
        "domain": "g",
        "date": "20230101",
        "param": "130",
        "expver": "0001",
        "step": range(0, 13, 6),
    }
    req = MarsRequest(
        "retrieve",
        **init_request,
    )
    second_request = {**init_request, **updates}
    req2 = MarsRequest(verb, **second_request)
    assert (req == req2) == expected



#!/usr/bin/env python3

# (C) Copyright 1996- ECMWF.

# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# Emanuele Danovaro - ECMWF May 2021

import sys
import os.path
import json
import datetime
from dateutil.parser import parse

def check_one(name, old, new):
    assert len(old) == 1 and len(new) == 1, f"{name.upper()}: too many values {len(old)} and {len(new)}"
    return old[0], new[0]

def check_atMostOne(name, old, new):
    assert len(old) < 2 and len(new) < 2, f"{name.upper()}: too many values {len(old)} and {len(new)}"
    return old[0] if 0 < len(old) else None, new[0] if 0 < len(new) else None

def compare_int(name, old, new):
    for o, n in zip(old, new):
        assert str(o).isnumeric() and str(n).isnumeric() and int(o) == int(n), f"{name.upper()}: value mismatch '{old}' and '{new}'"

def compare_float(name, old, new):
    for o, n in zip(old, new):
        try:
            o = float(o)
        except ValueError:
            o = o.lower()

        try:
            n = float(n)
        except ValueError:
            n = n.lower()

        assert o == n, f"{name.upper()}: value mismatch '{old}' and '{new}'"

def compare_intAndKeywords(name, old, new, keywords):
    o, n = check_atMostOne(name, old, new)
    if o in keywords or str(o).lower() in keywords:
        o = str(o).lower() if isinstance(o, str) else None
    if n in keywords or str(n).lower() in keywords:
        n = str(n).lower() if isinstance(n, str) else None
    assert o == n or (str(o).isnumeric() and str(n).isnumeric() and int(o) == int(n)), f"{name.upper()}: value mismatch '{old}' and '{new}'"

def compare_grid(name, old, new):
    for o, n in zip(old, new):
        try:
            o = float(o)
        except ValueError:
            o = o.lower()

        try:
            n = float(n)
        except ValueError:
            n = n.lower()
        assert o == n, f"{name.upper()}: value mismatch '{old}' and '{new}'"

def compare_expect(name, old, new):
    # 'any' or a positive integer value
    compare_intAndKeywords(name, old, new, ["any"])

def compare_accuracy(name, old, new):
    # 'off', 'av' or a positive integer value
    compare_intAndKeywords(name, old, new, [None, "off", "av", "n"])

def compare_frame(name, old, new):
    # 'off' or a positive integer value
    compare_intAndKeywords(name, old, new, ["off"])

def compare_truncation(name, old, new):
    # 'none', 'auto', 'off' or a positive integer value
    compare_intAndKeywords(name, old, new, ["off", "none", "auto"])

def compare_step(name, old, new):
    for o, n in zip(old, new):
        oParts = str(o).split('-')
        nParts = str(n).split('-')
        assert len(oParts) == len(nParts) and all(float(oi) == float(ni) for oi, ni in zip(oParts, nParts)), f"{name.upper()}: value mismatch '{old}' and '{new}'"

def compare_str(name, old, new):
    o, n = check_one(name, old, new)
    assert o.replace('"', '').lower() == n.replace('"', '').lower(), f"{name.upper()}: string mismatch '{old[0]}' and '{new[0]}'"

def compare_strList(name, old, new):
    for o, n in zip(old, new):
        compare_str(name, [o], [n])

def compare_strCaseSensitive(name, old, new):
    for o, n in zip(old, new):
        assert o.replace('"', '') == n.replace('"', ''), f"{name.upper()}: string mismatch '{old}' and '{new}'"

def compare_time(name, old, new):
    o, n = check_one(name, old, new)

    if isinstance(o, str):
        o = int(o.replace(':', ''))
    if o>9999:
        o = o/100

    if isinstance(n, str):
        n = int(n.replace(':', ''))
    if n > 9999:
        n = n / 100

    assert o == n, f"{name.upper()}: time mismatch '{old[0]}' and '{new[0]}'"

def compare_date(name, old, new):
    o, n = check_one(name, old, new)
    oStart = parse(str(o), default=datetime.datetime(1901, 1, 1))
    oEnd = parse(str(o), default=datetime.datetime(2001, 12, 31))
    nStart = parse(str(n), default=datetime.datetime(1901, 1, 1))
    nEnd = parse(str(n), default=datetime.datetime(2001, 12, 31))
    assert oStart.date() == nStart.date() and oEnd.date() == nEnd.date(), f"{name.upper()}: date mismatch '{old[0]}' and '{new[0]}'"

def compare_dates(name, old, new):
    for o, n in zip(old, new):
        oStart = parse(str(o), default=datetime.datetime(1901, 1, 1))
        oEnd = parse(str(o), default=datetime.datetime(2001, 12, 31))
        nStart = parse(str(n), default=datetime.datetime(1901, 1, 1))
        nEnd = parse(str(n), default=datetime.datetime(2001, 12, 31))
        assert oStart.date() == nStart.date() and oEnd.date() == nEnd.date(), f"{name.upper()}: date mismatch '{o}' and '{n}'"

def compare_expver(name, old, new):
    o, n = check_one(name, old, new)
    if isinstance(o, str):
        try:
            o = int(o)
        except ValueError:
            o = o.lower()

    if isinstance(n, str):
        try:
            n = int(n)
        except ValueError:
            n = n.lower()

    assert o == n, f"{name.upper()}: mismatch '{old[0]}' and '{new[0]}'"

def ignore(name, old, new):
    pass

def ignore_fixYaml(name, old, new):
    pass

comparators = {
    "accuracy": ignore_fixYaml, # ?????
    # "accuracy": compare_accuracy,
    "anoffset": compare_int, # ?????
    "area": ignore_fixYaml,
    "bitmap": compare_strCaseSensitive,
    # block
    # channel
    "class": compare_str,
    "database": compare_str, # ?????
    "date": compare_date,
    "diagnostic": compare_int, # ??????
    "direction": compare_int, # should we handle case "all"
    "domain": compare_str,
    "duplicates": compare_str, # can we ignore ?
    "expect": compare_expect,
    "expver": compare_expver,
    "fcmonth": compare_int,
    "fcperiod": compare_step, # ??????
    "fieldset": compare_strCaseSensitive,
    "filter": compare_str,
    "format": compare_str, # ?????
    "frame": compare_frame,
    "frequency": compare_int, # ??????
    "grid": compare_grid,
    "hdate": compare_dates,
    "ident": compare_int,
    "interpolation": compare_str,
    # "intgrid":
    "iteration": compare_int, # ??????
    "latitude": compare_float,
    "levelist": compare_float,
    "levtype": compare_str,
    "longitude": compare_float,
    "lsm": compare_str, # ?????
    "method": compare_int,
    "number": compare_int,
    "obsgroup": compare_str, # ?????
    "obstype": compare_int,
    "origin": compare_expver, # ?????
    "packing": compare_str,
    "padding": compare_int, # ??????
    "param": compare_float, # <-- to check that short names have been replaced, otherwise we can use compare_strList
    "priority": compare_int,
    "product": compare_str, # ?????
    "range": compare_int,
    "refdate": compare_date,
    "reference": compare_int,
    "reportype": compare_int,
    "repres": ignore,
    # resol
    "rotation": compare_grid,
    "section": compare_str,
    "source": compare_strCaseSensitive,
    "step": compare_step,
    "stream": compare_str,
    "system": compare_int,
    "target": compare_strCaseSensitive,
    "time": compare_time,
    "truncation": compare_truncation,
    "type": compare_str,
    "use": compare_str
}

def compare(name, old, new):
    if not isinstance(old, list):
        old = [old]
    if not isinstance(new, list):
        new = [new]

    assert len(old) == len(new), f"{name}: list mismatch {len(old)} and {len(new)}"
    assert name in comparators.keys(), f"{name}: not supported"
    comparators[name](name, old, new)

# compare_date('date', '20210501', '2021-05-01')
# compare_date('date', '20210501', '2021-May-01')
# compare_date('date', '20210501', '2021-may-01')
# compare_date('date', 'May-1', 'may-01')
# compare_date('date', 'May', 'may')
# compare_date('date', 'May', '20210521')

# check command line parameters
if len(sys.argv) != 3 or not os.path.isfile(sys.argv[1]) or not os.path.isfile(sys.argv[2]):
    print('Please specify the json files to be compared.')
    print('Usage: ',sys.argv[0], '<first.json> <second.json>')
    if len(sys.argv)>1 and not os.path.isfile(sys.argv[1]):
        print(sys.argv[1], 'is not a file')
    if len(sys.argv)>2 and not os.path.isfile(sys.argv[2]):
        print(sys.argv[2], 'is not a file')
    exit(1)

#open json files and build case insensitive dictionaries
json_lines_1 = []
with open(sys.argv[1]) as json_file:
    for line in json_file:
        j = {}
        for key, value in json.loads(line).items():
            j[key.lower()] = value
        json_lines_1.append(j)

json_lines_2 = []
with open(sys.argv[2]) as json_file:
    for line in json_file:
        j = {}
        for key, value in json.loads(line).items():
            j[key.lower()] = value
        json_lines_2.append(j)

assert len(json_lines_1) == len(json_lines_2), f"Json mismatch: {sys.argv[1]} contains {len(json_lines_1)} while {sys.argv[2]} contains {len(json_lines_2)}"

for j1, j2 in zip(json_lines_1, json_lines_2):
    for key in set(j1.keys()) | set(j2.keys()):
        compare(key, j1.get(key), j2.get(key))



# (c) Copyright 1996-2012 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# Documentation is available on Confluence
# https://confluence.ecmwf.int/display/METK/MetKit+configuration


import os
import sys
from collections import OrderedDict
import psycopg2
import yaml

DEFAULT_PORT = 5432

def main():
    try:
        HOST = os.environ['PRODUCT_CATALOGUE_HOST']
        DB = os.environ['PRODUCT_CATALOGUE_DB']
        USER = os.environ['PRODUCT_CATALOGUE_USER']
        PASSWORD = os.environ['PRODUCT_CATALOGUE_PASSWORD']
        PORT = (
            int(os.environ['PRODUCT_CATALOGUE_PORT'])
            if 'PRODUCT_CATALOGUE_PORT' in os.environ
            else DEFAULT_PORT
        )
    except KeyError as e:
        print("ERROR: Environment variable not found: {}".format(e))
        sys.exit(1)

    with psycopg2.connect(host=HOST, dbname=DB, user=USER, password=PASSWORD, port=PORT) as conn:
        with conn.cursor() as cur:
            cur.execute(
                "SELECT DISTINCT class, stream, type, levtype, param::INTEGER FROM fields WHERE param != '' ORDER BY class, stream, type, levtype, param::INTEGER"
            )
            rows = cur.fetchall()

    index = OrderedDict()
    for row in rows:
        cl, stream, type_, levtype, param = row
        key = (cl, stream, type_, levtype)
        if key not in index:
            index[key] = []
        index[key].append(param)
        
    # Manually add type=tf parameters for PGEN
    index[("od", "oper", "tf", "")] = [129, 999]
    index[("od", "scda", "tf", "")] = [129, 999]
    index[("od", "enfo", "tf", "")] = [129, 999]
    
    yaml_dump_data = []
    for key, vals in sorted(index.items()):
        if key[3]:
            yaml_dump_data.append(
                    [{"class":key[0], "stream":key[1], "type":key[2], "levtype":key[3]}, vals]
            )
        else:
            yaml_dump_data.append([{"class":key[0], "stream":key[1], "type":key[2]}, vals])

    with open("params.yaml", "w") as f:
        f.write(
            "# File automatically generated by %s\n# Do not edit\n\n"
            % (os.path.basename(__file__))
        )
        f.write(yaml.safe_dump(yaml_dump_data, default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import MySQLdb
import yaml
import re
import os


db = MySQLdb.connect(
    "grib-param-db-prod.ecmwf.int",
    os.environ['PARAM_DB_USER'],
    os.environ['PARAM_DB_PASSWORD'],
    "param")


cursor = db.cursor()


with open("params.yaml") as f:
    PARAMS = yaml.load(f.read())

for entry in PARAMS:
    when, parms = entry
    for p in parms:
        cursor.execute("select count(*) from param where id=%s" % p)
        for data in cursor.fetchall():
            if data[0] == 0:
                print("%s: %s is not in param DB" % (when, p))
